----------------------------------------------------------------------------------
-- Engineer:       Christoph Lubrich
-- 
-- Create Date:    09:18:52 07/08/2009 
-- Design Name: 
-- Module Name:    minilog - Behavioral 
-- Project Name:   minilog VHDL Test
-- Target Devices: 
-- Tool versions:  ISE 9.1
-- Description: 
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.numeric_std.ALL;
--use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity minilog is
	generic(addresswidth : integer := 12; datawidth : integer := 8);
    Port (  -- # Clock Signale #
	         clk_int      : in  STD_LOGIC;
				clk_80       : in  STD_LOGIC;
				-- Logisch 1 sezt die Zhler auf 0
				clear        : in  STD_LOGIC;
            clk_div      : in  STD_LOGIC_VECTOR (1 downto 0);
				
				-- # uC / LA Signale #
				in_data      : in  STD_LOGIC_VECTOR(datawidth-1 downto 0);
				trigger_mask : in  STD_LOGIC_VECTOR(datawidth-1 downto 0);
				trigger_data : in  STD_LOGIC_VECTOR(datawidth-1 downto 0);
				-- Logisch 1 schaltet den Buffer aus
				buffer_off   : in  STD_LOGIC;
				-- Logisch 1 fr lesen
				ram_read     : in  STD_LOGIC;
				-- Logisch 0 Trigger aktiv
				TRIGGER_N    : out  STD_LOGIC;
				
				-- # RAM Signale #
				RAM_ADDR   : out STD_LOGIC_VECTOR(addresswidth-1 downto 0);
				RAM_DATA   : out STD_LOGIC_VECTOR(datawidth-1 downto 0);
				RAM_WE     : out STD_LOGIC;
				RAM_OE     : out STD_LOGIC
				
				
	);
end minilog;

architecture Behavioral of minilog is
signal address : integer range 0 to 2**(addresswidth-1) := 0;
signal divider : integer range 0 to 3                   := 0;
signal countUP : std_logic := '0';
signal disable : std_logic := '0';
signal trigger : std_logic := '1';
signal mask    : std_logic_vector(datawidth-1 downto 0);

begin

	mask <= trigger_mask OR in_data;
	
	trigger <= '0' when mask = trigger_data else '1';

	TRIGGER_N <= '1' when ram_read = '1' else trigger;

	disable <= buffer_off OR ram_read;

	RAM_DATA <= (others=>'Z') when disable = '0' else in_data;

	RAMClockSource: process
	begin
		wait until rising_edge(clk_80);
		countUP <= '0';
		if clear = '1' then
			divider <= 0;
		else
			case clk_div is
				-- AVR gibt den Takt vor!
				when "00" => null; -- TODO !
				-- Takt/4
				when "01" => 
						if divider = 3 then
							divider <= 0;
							countUP <= '1';
						else
							divider <= divider + 1;
						end if;
				-- Takt/2
				when "10" =>
						if divider = 1 then
							divider <= 0;
							countUP <= '1';
						else
							divider <= divider + 1;
						end if;
				-- Takt ungeteilt
				when "11" =>
						divider <= 0;
						countUP <= '1';
				when others => null;
			end case;
		end if;
	end process;
	
	RAMCounter: process
	begin
		wait until rising_edge(clk_80);
		if clear = '1' then
			address <= 0;
		else
			RAM_ADDR <= STD_LOGIC_VECTOR(TO_UNSIGNED(address, addresswidth));
			if countUP = '1' then
				address <= address + 1;
			end if;
		end if;
	end process;
	
	-- RAM WE und Read Enable Signal
	RAMWE: process
	begin
		wait until rising_edge(clk_80);
		-- Default Inaktiv
		RAM_WE <= '1';
		RAM_OE <= '1';
		-- ram_read hat Vorrang
		if ram_read = '1' then
			RAM_OE <= '0';
		else
			if countUP = '1' then
				RAM_WE <= '0';
			end if;
		end if;
	end process;
	

end Behavioral;

